package com.ruoyi.ss.suit.service;

import com.ruoyi.ss.suit.domain.Suit;
import com.ruoyi.ss.suit.domain.SuitBO;
import com.ruoyi.ss.suit.domain.SuitQuery;
import com.ruoyi.ss.suit.domain.SuitVO;

import java.util.List;
import java.util.Map;
import java.util.function.Function;

/**
 * 套餐Service接口
 *
 * @author ruoyi
 * @date 2024-04-25
 */
public interface SuitService
{
    /**
     * 查询套餐
     *
     * @param suitId 套餐主键
     * @return 套餐
     */
    public SuitVO selectSuitBySuitId(Long suitId);

    /**
     * 查询套餐列表
     *
     * @param suit 套餐
     * @return 套餐集合
     */
    public List<SuitVO> selectSuitList(SuitQuery suit);

    /**
     * 新增套餐
     *
     * @param suit 套餐
     * @return 结果
     */
    public int insertSuit(SuitBO suit);

    /**
     * 修改套餐
     *
     * @param suit 套餐
     * @return 结果
     */
    public int updateSuit(SuitBO suit);

    /**
     * 批量删除套餐
     *
     * @param suitIds 需要删除的套餐主键集合
     * @return 结果
     */
    public int deleteSuitBySuitIds(Long[] suitIds);

    /**
     * 删除套餐信息
     *
     * @param suitId 套餐主键
     * @return 结果
     */
    public int deleteSuitBySuitId(Long suitId);

    /**
     * 批量逻辑删除
     * @param suitIds
     * @return
     */
    int logicDel(List<Long> suitIds);

    /**
     * 通用查询映射方法
     * @param query 查询条件
     * @param keyMapper 映射方法，需要保证唯一性
     */
    <K> Map<K, SuitVO> selectMap(SuitQuery query, Function<? super SuitVO, ? extends K> keyMapper);

    /**
     * 通用分组方法
     * @param query 查询条件
     * @param keyMapper 分组映射方法，需要保证唯一性
     */
    <K> Map<K, List<SuitVO>> selectGroup(SuitQuery query, Function<? super SuitVO, ? extends K> keyMapper);

    /**
     * 根据设备查询套餐列表
     * @param deviceId
     * @return
     */
    List<SuitVO> selectByDeviceId(Long deviceId);

    /**
     * 查询套餐数量
     * @param query
     * @return
     */
    int selectCount(SuitQuery query);

    /**
     * 根据设备ID删除套餐
     */
    int logicDelByDeviceId(Long deviceId);

    /**
     * 查询一个
     */
    SuitVO selectOne(SuitQuery query);

    /**
     * 设置排序
     */
    int setSort(List<? extends Suit> list);
}
