package com.ruoyi.web.controller.ss;

import java.util.List;
import javax.servlet.http.HttpServletResponse;

import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.ss.transactionBill.domain.TransactionBillQuery;
import com.ruoyi.ss.transactionBill.domain.vo.TransactionBillVO;
import com.ruoyi.ss.transactionBill.domain.dto.BillRefundDTO;
import com.ruoyi.ss.transactionBill.domain.dto.WithdrawApprovalDTO;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.ss.transactionBill.domain.TransactionBill;
import com.ruoyi.ss.transactionBill.service.TransactionBillService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 充值记录Controller
 *
 * @author 邱贞招
 * @date 2024-02-21
 */
@RestController
@RequestMapping("/system/bill")
public class SmTransactionBillController extends BaseController
{
    @Autowired
    private TransactionBillService smTransactionBillService;

    /**
     * 查询充值记录列表
     */
    @PreAuthorize("@ss.hasPermi('system:bill:list')")
    @GetMapping("/list")
    public TableDataInfo list(TransactionBillQuery smTransactionBill)
    {
        startPage();
        List<TransactionBillVO> list = smTransactionBillService.selectSmTransactionBillList(smTransactionBill);
        return getDataTable(list);
    }

    /**
     * 导出充值记录列表
     */
    @PreAuthorize("@ss.hasPermi('system:bill:export')")
    @Log(title = "充值记录", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TransactionBillQuery smTransactionBill)
    {
        List<TransactionBillVO> list = smTransactionBillService.selectSmTransactionBillList(smTransactionBill);
        ExcelUtil<TransactionBillVO> util = new ExcelUtil<TransactionBillVO>(TransactionBillVO.class);
        util.exportExcel(response, list, "充值记录数据");
    }

    /**
     * 获取充值记录详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:bill:query')")
    @GetMapping(value = "/{billId}")
    public AjaxResult getInfo(@PathVariable("billId") Long billId)
    {
        return success(smTransactionBillService.selectSmTransactionBillByBillId(billId));
    }

    /**
     * 新增充值记录
     */
    @PreAuthorize("@ss.hasPermi('system:bill:add')")
    @Log(title = "充值记录", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TransactionBill transactionBill)
    {
        return toAjax(smTransactionBillService.insertSmTransactionBill(transactionBill));
    }

    /**
     * 修改充值记录
     */
    @PreAuthorize("@ss.hasPermi('system:bill:edit')")
    @Log(title = "充值记录", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TransactionBill transactionBill)
    {
        return toAjax(smTransactionBillService.updateSmTransactionBill(transactionBill));
    }

    /**
     * 删除充值记录
     */
    @PreAuthorize("@ss.hasPermi('system:bill:remove')")
    @Log(title = "充值记录", businessType = BusinessType.DELETE)
	@DeleteMapping("/{billIds}")
    public AjaxResult remove(@PathVariable Long[] billIds)
    {
        return toAjax(smTransactionBillService.deleteSmTransactionBillByBillIds(billIds));
    }

    @PreAuthorize("@ss.hasPermi('system:bill:query')")
    @GetMapping("/count")
    public AjaxResult count(TransactionBillQuery dto)
    {
        return success(smTransactionBillService.selectCount(dto));
    }

    @ApiOperation("同意提现申请")
    @PreAuthorize("@ss.hasPermi('system:bill:approval')")
    @PutMapping("/withdraw/pass")
    @Log(title = "同意提现申请", businessType = BusinessType.UPDATE)
    public AjaxResult passWithdraw(@RequestBody @Validated WithdrawApprovalDTO dto)
    {
        return success(smTransactionBillService.passWithdraw(dto));
    }

    @ApiOperation("拒绝提现申请")
    @PreAuthorize("@ss.hasPermi('system:bill:approval')")
    @PutMapping("/withdraw/reject")
    @Log(title = "拒绝提现申请", businessType = BusinessType.UPDATE)
    public AjaxResult rejectWithdraw(@RequestBody @Validated WithdrawApprovalDTO dto)
    {
        return success(smTransactionBillService.rejectWithdraw(dto));
    }

//    @ApiOperation("提现打款")
//    @PreAuthorize("@ss.hasPermi('system:bill:pay')")
//    @PutMapping("/withdraw/pay/{billId}")
//    @Log(title = "提现打款", businessType = BusinessType.UPDATE)
//    public AjaxResult payWithdraw(@PathVariable Long billId)
//    {
//        return success(smTransactionBillService.payWithdraw(billId));
//    }

    @ApiModelProperty("刷新支付结果")
    @PreAuthorize("@ss.hasPermi('system:bill:edit')")
    @GetMapping("/refreshPayResult/{billId}")
    @Log(title = "刷新支付结果", businessType = BusinessType.UPDATE)
    public AjaxResult refreshPayResult(@PathVariable Long billId)
    {
        smTransactionBillService.refreshPayResult(billId);
        return success();
    }

    @ApiModelProperty("手动设备充值")
    @GetMapping("/rechargeDevice/{billId}")
    @Log(title = "手动设备充值", businessType = BusinessType.UPDATE)
    public AjaxResult rechargeDevice(@PathVariable Long billId) {
        return toAjax(smTransactionBillService.rechargeDevice(billId));
    }

    // 订单退款
    @PutMapping("/refund")
    @PreAuthorize("@ss.hasPermi('system:bill:refund')")
    public AjaxResult refund(@RequestBody @Validated BillRefundDTO dto) {
        LoginUser loginUser = getLoginUser();
        dto.setUserName(loginUser.getUsername());
        dto.setUserType(loginUser.getLoginType().getType());
        dto.setUserId(getUserId());
        return toAjax(smTransactionBillService.refund(dto));
    }
}
